import processing.opengl.*;


import peasy.*;

PeasyCam cam;

int nr = 100;
int fw = 500;
int fh = 500;

int rows = 8;
int colums = 14; 
int tileW = 43;
int tileH = 43;
Brick [][] bricks;

PImage img;

float count = 0;
void setup(){
  size(800,600,OPENGL);

  cam = new PeasyCam(this, 800);
  cam.setMinimumDistance(5);
  cam.setMaximumDistance(1000);

  img = loadImage("gridtex.png");

  bricks = new Brick[colums][rows];
  for(int i=0; i<colums; i++){
    for(int j=0; j<rows; j++){
      bricks[i][j] = new Brick(i*tileW,j*tileH,0,tileW,tileH, 2*tileW,i*tileW, j*tileH);
    }
  }
  
  frameRate(30);
}


void draw(){
  background(255);
  lights();
  drawGrid(10,10);
  drawCoords(10);
  textureMode(IMAGE);
  
  count+=0.1;
  calcZValues(count,-50,50);

noStroke();
  beginShape(QUADS);
  texture(img);
  for(int i=0; i<colums; i++){
    for(int j=0; j<rows; j++){
       bricks[i][j].render();
    }
  }
  endShape();
 

}

void calcZValues(float step,float minVal, float maxVal){
  for(int i=0; i<colums; i++){
    for(int j=0; j<rows; j++){
      float val = sin(i+step)+cos(j+step);
      bricks[i][j].z =map(val,-2,2,minVal,maxVal);
    }
  }
  
}






void drawGrid(float cells, float cellSide ){

  stroke(0);
  float w = cells*cellSide;
  pushMatrix();
  translate(-w/2,0,-w/2);
  for(int i=0; i<cells; i++){

    beginShape();
    vertex(i*cellSide,0,0);
    vertex(i*cellSide,0,cells*cellSide);
    endShape();
    //line(i*cellSide,0,i*cellSide,w*cellSide);

  }


  popMatrix();
  pushMatrix();
  translate(-w/2,0,-w/2);
  for(int j=0; j<cells; j++){
    beginShape();
    vertex(0,0,j*cellSide);
    vertex(cells*cellSide,0,j*cellSide);
    endShape();
  }
  popMatrix();

}


void drawCoords(float len){

  strokeWeight(3);
  stroke(0,0,255);
  beginShape(LINES);
  vertex(0,0,0);
  vertex(1*len,0*len,0*len);
  endShape();

  stroke(255,0,0);
  beginShape(LINES);
  vertex(0,0,0);
  vertex(0*len,1*len,0*len);
  endShape();

  stroke(0,255,0);
  beginShape(LINES);
  vertex(0,0,0);
  vertex(0*len,0*len,1*len);
  endShape();
  strokeWeight(1);
  stroke(0);

}

class Brick{

  float x,y,z,w,h,d,tx,ty;

  public Brick(int x, int y, int z,int w,int h, int d,int tx, int ty){
    this.x = x; 
    this.y = y;
    this.z = z;
    this.w = w; 
    this.h = h; 
    this.d = -d;
    this.tx = tx;
    this.ty = ty;
  }
  
  void render(){
    pushMatrix();
    translate(x,y,z);
    
    //textured face
    fill(255);
     textureMode(IMAGE);
    beginShape(QUADS);
    texture(img);
    vertex(0,0,0,tx,ty);
    vertex(w,0,0,tx+tileW,ty);
    vertex(w,h,0,tx+tileW,ty+tileH);
    vertex(0,h,0,tx,ty+tileH);
    endShape();
    
    //non textured sides
    fill(200);
     beginShape(QUADS);
    vertex(0,0,0);
    vertex(0,0,d);
    vertex(w,0,d);
    vertex(w,0,0);
    
    vertex(0,h,0);
    vertex(0,h,d);
    vertex(w,h,d);
    vertex(w,h,0);
    
    vertex(0,0,0);
    vertex(0,h,0);
    vertex(0,h,d);
    vertex(0,0,d);
    
    vertex(w,0,0);
    vertex(w,0,d);
    vertex(w,h,d);
    vertex(w,h,0);
    
    vertex(0,0,d);
    vertex(w,0,d);
    vertex(w,h,d);
    vertex(0,h,d);
    
    endShape();
    
    popMatrix();
  }
}







